@@ -1,5 +1,5 @@
-use crate::algebra::{Matrix4, Vector3};
 use crate::{
+    algebra::{Matrix4, Vector3},
     math::{aabb::AxisAlignedBoundingBox, plane::Plane},
     visitor::{Visit, VisitResult, Visitor},
 };
@@ -12,6 +12,7 @@ pub struct Frustum {
 }
 
 impl Default for Frustum {
+    #[inline]
     fn default() -> Self {
         Self::from(Matrix4::new_perspective(
             1.0,
@@ -24,6 +25,7 @@ impl Default for Frustum {
 }
 
 impl Frustum {
+    #[inline]
     pub fn from(m: Matrix4<f32>) -> Option<Self> {
         Some(Self {
             planes: [
@@ -112,6 +114,21 @@ impl Frustum {
         self.right().intersection_point(self.top(), self.near())
     }
 
+    #[inline]
+    pub fn corners(&self) -> [Vector3<f32>; 8] {
+        [
+            self.left_top_front_corner(),
+            self.left_bottom_front_corner(),
+            self.right_bottom_front_corner(),
+            self.right_top_front_corner(),
+            self.left_top_back_corner(),
+            self.left_bottom_back_corner(),
+            self.right_bottom_back_corner(),
+            self.right_top_back_corner(),
+        ]
+    }
+
+    #[inline]
     pub fn is_intersects_point_cloud(&self, points: &[Vector3<f32>]) -> bool {
         for plane in self.planes.iter() {
             let mut back_points = 0;
@@ -128,6 +145,7 @@ impl Frustum {
         true
     }
 
+    #[inline]
     pub fn is_intersects_aabb(&self, aabb: &AxisAlignedBoundingBox) -> bool {
         let corners = [
             Vector3::new(aabb.min.x, aabb.min.y, aabb.min.z),
@@ -143,6 +161,7 @@ impl Frustum {
         self.is_intersects_point_cloud(&corners)
     }
 
+    #[inline]
     pub fn is_intersects_aabb_offset(
         &self,
         aabb: &AxisAlignedBoundingBox,
@@ -162,6 +181,7 @@ impl Frustum {
         self.is_intersects_point_cloud(&corners)
     }
 
+    #[inline]
     pub fn is_intersects_aabb_transform(
         &self,
         aabb: &AxisAlignedBoundingBox,
@@ -205,6 +225,7 @@ impl Frustum {
         self.is_intersects_point_cloud(&corners)
     }
 
+    #[inline]
     pub fn is_contains_point(&self, pt: Vector3<f32>) -> bool {
         for plane in self.planes.iter() {
             if plane.dot(&pt) <= 0.0 {
@@ -214,6 +235,7 @@ impl Frustum {
         true
     }
 
+    #[inline]
     pub fn is_intersects_sphere(&self, p: Vector3<f32>, r: f32) -> bool {
         for plane in self.planes.iter() {
             let d = plane.dot(&p);
