@@ -1,32 +1,38 @@
-//! Pool is contiguous block of memory with fixed-size entries, each entry can be
-//! either vacant or occupied. When you put an object into pool you get handle to
+//! A generational arena - a contiguous growable array type which allows removing
+//! from the middle without shifting and therefore without invalidating other indices.
+//!
+//! Pool is a contiguous block of memory with fixed-size entries, each entry can be
+//! either vacant or occupied. When you put an object into the pool you get a handle to
 //! that object. You can use that handle later on to borrow a reference to an object.
-//! Handle can point to some object or be invalid, this may look similar to raw
+//! A handle can point to some object or be invalid, this may look similar to raw
 //! pointers, but there is two major differences:
 //!
-//! 1) We can check if handle is valid before accessing object it might point to.
-//! 2) We can ensure that handle we using still valid for an object it points to.
-//! Each handle store special field that is shared across entry and handle, so
-//! handle is valid if these field in both handle and entry are same. This protects
-//! from situations where you have a handle that has valid index of a record, but
-//! payload in this record was replaced.
+//! 1) We can check if a handle is valid before accessing the object it might point to.
+//! 2) We can ensure the handle we're using is still valid for the object it points to
+//! to make sure it hasn't been replaced with a different object on the same position.
+//! Each handle stores a special field called generation which is shared across the entry
+//! and the handle, so the handle is valid if these fields are the same on both the entry
+//! and the handle. This protects from situations where you have a handle that has
+//! a valid index of a record, but the payload in this record has been replaced.
 //!
-//! Contiguous memory block increases efficiency of memory operations - CPU will
+//! Contiguous memory block increases efficiency of memory operations - the CPU will
 //! load portions of data into its cache piece by piece, it will be free from any
-//! indirections that might cause cache invalidation. This is so called cache
+//! indirections that might cause cache invalidation. This is the so called cache
 //! friendliness.
 
 #![allow(clippy::unneeded_field_pattern)]
 
-use crate::visitor::{Visit, VisitResult, Visitor};
+use crate::{
+    inspect::{Inspect, PropertyInfo},
+    visitor::{Visit, VisitResult, Visitor},
+};
 use std::{
-    fmt::{Debug, Formatter},
+    any::TypeId,
+    fmt::{Debug, Display, Formatter},
+    future::Future,
     hash::{Hash, Hasher},
     iter::FromIterator,
     marker::PhantomData,
-};
-use std::{
-    future::Future,
     ops::{Index, IndexMut},
 };
 
@@ -61,16 +67,55 @@ pub struct Handle<T> {
     type_marker: PhantomData<T>,
 }
 
+impl<T: 'static> Inspect for Handle<T> {
+    fn properties(&self) -> Vec<PropertyInfo<'_>> {
+        vec![
+            PropertyInfo {
+                owner_type_id: TypeId::of::<Self>(),
+                name: "index",
+                display_name: "Index",
+                value: &self.index,
+                read_only: true,
+                min_value: None,
+                max_value: None,
+                step: None,
+                precision: None,
+                description: "Index of an object in a pool.".to_string(),
+            },
+            PropertyInfo {
+                owner_type_id: TypeId::of::<Self>(),
+                name: "generation",
+                display_name: "Generation",
+                value: &self.generation,
+                read_only: true,
+                min_value: None,
+                max_value: None,
+                step: None,
+                precision: None,
+                description: "Generation of an object in a pool.".to_string(),
+            },
+        ]
+    }
+}
+
 unsafe impl<T> Send for Handle<T> {}
 unsafe impl<T> Sync for Handle<T> {}
 
+impl<T> Display for Handle<T> {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        write!(f, "{}:{}", self.index, self.generation)
+    }
+}
+
 /// Type-erased handle.
-#[derive(Copy, Clone, Debug, Ord, PartialOrd, PartialEq, Eq, Hash)]
+#[derive(Copy, Clone, Debug, Ord, PartialOrd, PartialEq, Eq, Hash, Inspect)]
 pub struct ErasedHandle {
     /// Index of object in pool.
+    #[inspect(read_only)]
     index: u32,
     /// Generation number, if it is same as generation of pool record at
     /// index of handle then this is valid handle.
+    #[inspect(read_only)]
     generation: u32,
 }
 
@@ -321,26 +366,137 @@ impl<T> Pool<T> {
     }
 
     #[inline]
-    pub fn with_capacity(capacity: usize) -> Self {
+    pub fn with_capacity(capacity: u32) -> Self {
+        let capacity = usize::try_from(capacity).expect("capacity overflowed usize");
         Pool {
             records: Vec::with_capacity(capacity),
             free_stack: Vec::new(),
         }
     }
 
+    fn records_len(&self) -> u32 {
+        u32::try_from(self.records.len()).expect("Number of records overflowed u32")
+    }
+
+    fn records_get(&self, index: u32) -> Option<&PoolRecord<T>> {
+        let index = usize::try_from(index).expect("Index overflowed usize");
+        self.records.get(index)
+    }
+
+    fn records_get_mut(&mut self, index: u32) -> Option<&mut PoolRecord<T>> {
+        let index = usize::try_from(index).expect("Index overflowed usize");
+        self.records.get_mut(index)
+    }
+
     #[inline]
     #[must_use]
     pub fn spawn(&mut self, payload: T) -> Handle<T> {
         self.spawn_with(|_| payload)
     }
 
+    /// Tries to put an object in the pool at given position. Returns `Err(payload)` if a corresponding
+    /// entry is occupied.
+    ///
+    /// # Performance
+    ///
+    /// The method has O(n) complexity in worst case, where `n` - amount of free records in the pool.
+    /// In typical uses cases `n` is very low. It should be noted that if a pool is filled entirely
+    /// and you trying to put an object at the end of pool, the method will have O(1) complexity.
+    ///
+    /// # Panics
+    ///
+    /// Panics if the index is occupied or reserved (e.g. by [`take_reserve`]).
+    ///
+    /// [`take_reserve`]: Pool::take_reserve
+    #[inline]
+    pub fn spawn_at(&mut self, index: u32, payload: T) -> Result<Handle<T>, T> {
+        self.spawn_at_internal(index, INVALID_GENERATION, payload)
+    }
+
+    /// Tries to put an object in the pool at given handle. Returns `Err(payload)` if a corresponding
+    /// entry is occupied.
+    ///
+    /// # Performance
+    ///
+    /// The method has O(n) complexity in worst case, where `n` - amount of free records in the pool.
+    /// In typical uses cases `n` is very low. It should be noted that if a pool is filled entirely
+    /// and you trying to put an object at the end of pool, the method will have O(1) complexity.
+    ///
+    /// # Panics
+    ///
+    /// Panics if the index is occupied or reserved (e.g. by [`take_reserve`]).
+    ///
+    /// [`take_reserve`]: Pool::take_reserve
+    pub fn spawn_at_handle(&mut self, handle: Handle<T>, payload: T) -> Result<Handle<T>, T> {
+        self.spawn_at_internal(handle.index, handle.generation, payload)
+    }
+
+    fn spawn_at_internal(
+        &mut self,
+        index: u32,
+        desired_generation: u32,
+        payload: T,
+    ) -> Result<Handle<T>, T> {
+        let index_usize = usize::try_from(index).expect("index overflowed usize");
+        match self.records.get_mut(index_usize) {
+            Some(record) => match record.payload {
+                Some(_) => Err(payload),
+                None => {
+                    let position = self
+                        .free_stack
+                        .iter()
+                        .rposition(|i| *i == index)
+                        .expect("free_stack must contain the index of the empty record (most likely attempting to spawn at a reserved index)!");
+
+                    self.free_stack.remove(position);
+
+                    let generation = if desired_generation == INVALID_GENERATION {
+                        record.generation + 1
+                    } else {
+                        desired_generation
+                    };
+
+                    record.generation = generation;
+                    record.payload = Some(payload);
+
+                    Ok(Handle::new(index, generation))
+                }
+            },
+            None => {
+                // Spawn missing records to fill gaps.
+                for i in self.records_len()..index {
+                    self.records.push(PoolRecord {
+                        generation: 1,
+                        payload: None,
+                    });
+                    self.free_stack.push(i);
+                }
+
+                let generation = if desired_generation == INVALID_GENERATION {
+                    1
+                } else {
+                    desired_generation
+                };
+
+                self.records.push(PoolRecord {
+                    generation,
+                    payload: Some(payload),
+                });
+
+                Ok(Handle::new(index, generation))
+            }
+        }
+    }
+
     #[inline]
     #[must_use]
     /// Construct a value with the handle it would be given.
     /// Note: Handle is _not_ valid until function has finished executing.
     pub fn spawn_with<F: FnOnce(Handle<T>) -> T>(&mut self, callback: F) -> Handle<T> {
-        if let Some(free_index) = self.free_stack.last() {
-            let record = &mut self.records[*free_index as usize];
+        if let Some(free_index) = self.free_stack.pop() {
+            let record = self
+                .records_get_mut(free_index)
+                .expect("free stack contained invalid index");
 
             if record.payload.is_some() {
                 panic!(
@@ -351,16 +507,13 @@ impl<T> Pool<T> {
 
             let generation = record.generation + 1;
             let handle = Handle {
-                index: *free_index,
+                index: free_index,
                 generation,
                 type_marker: PhantomData,
             };
 
             let payload = callback(handle);
 
-            // Pop the index we've decided to use off.
-            self.free_stack.pop();
-
             record.generation = generation;
             record.payload.replace(payload);
             handle
@@ -395,8 +548,10 @@ impl<T> Pool<T> {
         F: FnOnce(Handle<T>) -> Fut,
         Fut: Future<Output = T>,
     {
-        if let Some(free_index) = self.free_stack.last() {
-            let record = &mut self.records[*free_index as usize];
+        if let Some(free_index) = self.free_stack.pop() {
+            let record = self
+                .records_get_mut(free_index)
+                .expect("free stack contained invalid index");
 
             if record.payload.is_some() {
                 panic!(
@@ -407,16 +562,13 @@ impl<T> Pool<T> {
 
             let generation = record.generation + 1;
             let handle = Handle {
-                index: *free_index,
+                index: free_index,
                 generation,
                 type_marker: PhantomData,
             };
 
             let payload = callback(handle).await;
 
-            // Pop the index we've decided to use off.
-            self.free_stack.pop();
-
             record.generation = generation;
             record.payload.replace(payload);
             handle
@@ -453,7 +605,7 @@ impl<T> Pool<T> {
     #[inline]
     #[must_use]
     pub fn borrow(&self, handle: Handle<T>) -> &T {
-        if let Some(record) = self.records.get(handle.index as usize) {
+        if let Some(record) = self.records_get(handle.index) {
             if record.generation == handle.generation {
                 if let Some(ref payload) = record.payload {
                     payload
@@ -462,7 +614,7 @@ impl<T> Pool<T> {
                 }
             } else {
                 panic!(
-                    "Attempt to use dangling handle {:?}. Record has {} generation!",
+                    "Attempt to use dangling handle {:?}. Record has generation {}!",
                     handle, record.generation
                 );
             }
@@ -496,7 +648,7 @@ impl<T> Pool<T> {
     #[must_use]
     pub fn borrow_mut(&mut self, handle: Handle<T>) -> &mut T {
         let record_count = self.records.len();
-        if let Some(record) = self.records.get_mut(handle.index as usize) {
+        if let Some(record) = self.records_get_mut(handle.index) {
             if record.generation == handle.generation {
                 if let Some(ref mut payload) = record.payload {
                     payload
@@ -522,7 +674,7 @@ impl<T> Pool<T> {
     #[inline]
     #[must_use]
     pub fn try_borrow(&self, handle: Handle<T>) -> Option<&T> {
-        self.records.get(handle.index as usize).and_then(|r| {
+        self.records_get(handle.index).and_then(|r| {
             if r.generation == handle.generation {
                 r.payload.as_ref()
             } else {
@@ -539,7 +691,7 @@ impl<T> Pool<T> {
     #[inline]
     #[must_use]
     pub fn try_borrow_mut(&mut self, handle: Handle<T>) -> Option<&mut T> {
-        self.records.get_mut(handle.index as usize).and_then(|r| {
+        self.records_get_mut(handle.index).and_then(|r| {
             if r.generation == handle.generation {
                 r.payload.as_mut()
             } else {
@@ -688,15 +840,15 @@ impl<T> Pool<T> {
         (first, None)
     }
 
-    /// Moves object out of pool using given handle. All handles to the object will become invalid.
+    /// Moves object out of the pool using the given handle. All handles to the object will become invalid.
     ///
     /// # Panics
     ///
-    /// Panics if given handle is invalid.
-    ///
+    /// Panics if the given handle is invalid.
     #[inline]
     pub fn free(&mut self, handle: Handle<T>) -> T {
-        if let Some(record) = self.records.get_mut(handle.index as usize) {
+        let index = usize::try_from(handle.index).expect("index overflowed usize");
+        if let Some(record) = self.records.get_mut(index) {
             if record.generation == handle.generation {
                 // Remember this index as free
                 self.free_stack.push(handle.index);
@@ -717,28 +869,29 @@ impl<T> Pool<T> {
         }
     }
 
-    /// Moves object out of pool using given handle with a promise that object will be returned back.
-    /// Returns pair (ticket, value). Ticket must be used to put value back!
+    /// Moves an object out of the pool using the given handle with a promise that the object will be returned back.
+    /// Returns pair (ticket, value). The ticket must be used to put the value back!
     ///
     /// # Motivation
     ///
     /// This method is useful when you need to take temporary ownership of an object, do something
-    /// with it and then put it back while keep all handles valid and be able to put new objects into
-    /// pool without overriding a payload at its handle.
+    /// with it and then put it back while preserving all handles to it and being able to put new objects into
+    /// the pool without overriding the payload at its handle.
     ///
     /// # Notes
     ///
-    /// All handles to the object will be invalid until object is returned in pool! Pool record will
-    /// be reserved for further put_back call, which means if you lose ticket you will have empty
+    /// All handles to the object will be temporarily invalid until the object is returned to the pool! The pool record will
+    /// be reserved for a further [`put_back`] call, which means if you lose the ticket you will have an empty
     /// "unusable" pool record forever.
     ///
     /// # Panics
     ///
-    /// Panics if given handle is invalid.
+    /// Panics if the given handle is invalid.
     ///
+    /// [`put_back`]: Pool::put_back
     #[inline]
     pub fn take_reserve(&mut self, handle: Handle<T>) -> (Ticket<T>, T) {
-        if let Some(record) = self.records.get_mut(handle.index as usize) {
+        if let Some(record) = self.records_get_mut(handle.index) {
             if record.generation == handle.generation {
                 if let Some(payload) = record.payload.take() {
                     let ticket = Ticket {
@@ -763,10 +916,12 @@ impl<T> Pool<T> {
         }
     }
 
-    /// Does same as take_reserve but returns option, instead of panic.
+    /// Does the same as [`take_reserve`] but returns an option, instead of panicking.
+    ///
+    /// [`take_reserve`]: Pool::take_reserve
     #[inline]
     pub fn try_take_reserve(&mut self, handle: Handle<T>) -> Option<(Ticket<T>, T)> {
-        if let Some(record) = self.records.get_mut(handle.index as usize) {
+        if let Some(record) = self.records_get_mut(handle.index) {
             if record.generation == handle.generation {
                 if let Some(payload) = record.payload.take() {
                     let ticket = Ticket {
@@ -784,13 +939,15 @@ impl<T> Pool<T> {
             None
         }
     }
-    /// Returns value back into pool using given ticket.
+
+    /// Returns the value back into the pool using the given ticket. See [`take_reserve`] for more
+    /// information.
     ///
-    /// # Panics
-    ///
-    /// In normal conditions it must never panic.
+    /// [`take_reserve`]: Pool::take_reserve
     pub fn put_back(&mut self, ticket: Ticket<T>, value: T) -> Handle<T> {
-        let record = &mut self.records[ticket.index as usize];
+        let record = self
+            .records_get_mut(ticket.index)
+            .expect("Ticket index was invalid");
         let old = record.payload.replace(value);
         assert!(old.is_none());
         Handle::new(ticket.index, record.generation)
@@ -806,8 +963,8 @@ impl<T> Pool<T> {
     /// Returns total capacity of pool. Capacity has nothing about real amount of objects in pool!
     #[inline]
     #[must_use]
-    pub fn get_capacity(&self) -> usize {
-        self.records.len()
+    pub fn get_capacity(&self) -> u32 {
+        u32::try_from(self.records.len()).expect("records.len() overflowed u32")
     }
 
     /// Destroys all objects in pool. All handles to objects will become invalid.
@@ -817,7 +974,6 @@ impl<T> Pool<T> {
     /// Use this method cautiously if objects in pool have cross "references" (handles)
     /// to each other. This method will make all produced handles invalid and any further
     /// calls for [`borrow`](Self::borrow) or [`borrow_mut`](Self::borrow_mut) will raise panic.
-    ///
     #[inline]
     pub fn clear(&mut self) {
         self.records.clear();
@@ -826,28 +982,34 @@ impl<T> Pool<T> {
 
     #[inline]
     #[must_use]
-    pub fn at_mut(&mut self, n: usize) -> Option<&mut T> {
-        self.records.get_mut(n).and_then(|rec| rec.payload.as_mut())
+    pub fn at_mut(&mut self, n: u32) -> Option<&mut T> {
+        self.records_get_mut(n).and_then(|rec| rec.payload.as_mut())
     }
 
     #[inline]
     #[must_use]
-    pub fn at(&self, n: usize) -> Option<&T> {
-        self.records.get(n).and_then(|rec| rec.payload.as_ref())
+    pub fn at(&self, n: u32) -> Option<&T> {
+        self.records_get(n).and_then(|rec| rec.payload.as_ref())
     }
 
     #[inline]
     #[must_use]
-    pub fn handle_from_index(&self, n: usize) -> Handle<T> {
-        if let Some(record) = self.records.get(n) {
+    pub fn handle_from_index(&self, n: u32) -> Handle<T> {
+        if let Some(record) = self.records_get(n) {
             if record.generation != INVALID_GENERATION {
-                return Handle::new(n as u32, record.generation);
+                return Handle::new(n, record.generation);
             }
         }
         Handle::NONE
     }
 
-    /// Returns exact amount of "alive" objects in pool.
+    /// Returns the exact number of "alive" objects in the pool.
+    ///
+    /// Records that have been reserved (e.g. by [`take_reserve`]) are *not* counted.
+    ///
+    /// It iterates through the entire pool to count the live objects so the complexity is `O(n)`.
+    ///
+    /// See also [`total_count`].
     ///
     /// # Example
     ///
@@ -858,15 +1020,35 @@ impl<T> Pool<T> {
     /// pool.spawn(321);
     /// assert_eq!(pool.alive_count(), 2);
     /// ```
+    ///
+    /// [`take_reserve`]: Pool::take_reserve
+    /// [`total_count`]: Pool::total_count
     #[inline]
     #[must_use]
-    pub fn alive_count(&self) -> usize {
-        self.iter().count()
+    pub fn alive_count(&self) -> u32 {
+        let cnt = self.iter().count();
+        u32::try_from(cnt).expect("alive_count overflowed u32")
+    }
+
+    /// Returns the number of allocated objects in the pool.
+    ///
+    /// It also counts records that have been reserved (e.g. by [`take_reserve`]).
+    ///
+    /// This method is `O(1)`.
+    ///
+    /// See also [`alive_count`].
+    ///
+    /// [`take_reserve`]: Pool::take_reserve
+    /// [`alive_count`]: Pool::alive_count
+    pub fn total_count(&self) -> u32 {
+        let free = u32::try_from(self.free_stack.len()).expect("free stack length overflowed u32");
+        self.records_len() - free
     }
 
     #[inline]
     pub fn replace(&mut self, handle: Handle<T>, payload: T) -> Option<T> {
-        if let Some(record) = self.records.get_mut(handle.index as usize) {
+        let index_usize = usize::try_from(handle.index).expect("index overflowed usize");
+        if let Some(record) = self.records.get_mut(index_usize) {
             if record.generation == handle.generation {
                 self.free_stack.retain(|i| *i != handle.index);
 
@@ -891,7 +1073,7 @@ impl<T> Pool<T> {
     /// ```
     #[inline]
     pub fn is_valid_handle(&self, handle: Handle<T>) -> bool {
-        if let Some(record) = self.records.get(handle.index as usize) {
+        if let Some(record) = self.records_get(handle.index) {
             record.payload.is_some() && record.generation == handle.generation
         } else {
             false
@@ -1011,7 +1193,9 @@ impl<T> Pool<T> {
             let record_size = std::mem::size_of::<PoolRecord<T>>();
             let record_location = (val - offset_of!(PoolRecord<T>, payload)) - begin;
             if record_location % record_size == 0 {
-                return self.handle_from_index(record_location / record_size);
+                let index = record_location / record_size;
+                let index = u32::try_from(index).expect("Index overflowed u32");
+                return self.handle_from_index(index);
             }
         }
         Handle::NONE
@@ -1022,8 +1206,11 @@ impl<T> FromIterator<T> for Pool<T> {
     fn from_iter<C: IntoIterator<Item = T>>(iter: C) -> Self {
         let iter = iter.into_iter();
         let (lower_bound, upper_bound) = iter.size_hint();
+        let lower_bound = u32::try_from(lower_bound).expect("lower_bound overflowed u32");
+        let upper_bound =
+            upper_bound.map(|b| u32::try_from(b).expect("upper_bound overflowed u32"));
         let mut pool = Self::with_capacity(upper_bound.unwrap_or(lower_bound));
-        for v in iter.into_iter() {
+        for v in iter {
             let _ = pool.spawn(v);
         }
         pool
@@ -1044,6 +1231,24 @@ impl<T> IndexMut<Handle<T>> for Pool<T> {
     }
 }
 
+impl<'a, T> IntoIterator for &'a Pool<T> {
+    type Item = &'a T;
+    type IntoIter = PoolIterator<'a, T>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        self.iter()
+    }
+}
+
+impl<'a, T> IntoIterator for &'a mut Pool<T> {
+    type Item = &'a mut T;
+    type IntoIter = PoolIteratorMut<'a, T>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        self.iter_mut()
+    }
+}
+
 pub struct PoolIterator<'a, T> {
     ptr: *const PoolRecord<T>,
     end: *const PoolRecord<T>,
@@ -1150,7 +1355,7 @@ impl<'a, T> Iterator for PoolPairIteratorMut<'a, T> {
 
 #[cfg(test)]
 mod test {
-    use crate::pool::{Pool, INVALID_GENERATION};
+    use crate::pool::{Handle, Pool, INVALID_GENERATION};
 
     #[test]
     fn pool_sanity_tests() {
@@ -1158,15 +1363,15 @@ mod test {
         let foobar_handle = pool.spawn(String::from("Foobar"));
         assert_eq!(foobar_handle.index, 0);
         assert_ne!(foobar_handle.generation, INVALID_GENERATION);
-        let foobar_handle_copy = foobar_handle.clone();
+        let foobar_handle_copy = foobar_handle;
         assert_eq!(foobar_handle.index, foobar_handle_copy.index);
         assert_eq!(foobar_handle.generation, foobar_handle_copy.generation);
         let baz_handle = pool.spawn(String::from("Baz"));
         assert_eq!(pool.borrow(foobar_handle), "Foobar");
         assert_eq!(pool.borrow(baz_handle), "Baz");
         pool.free(foobar_handle);
-        assert_eq!(pool.is_valid_handle(foobar_handle_copy), false);
-        assert_eq!(pool.is_valid_handle(baz_handle), true);
+        assert!(!pool.is_valid_handle(foobar_handle_copy));
+        assert!(pool.is_valid_handle(baz_handle));
         let at_foobar_index = pool.spawn(String::from("AtFoobarIndex"));
         assert_eq!(at_foobar_index.index, 0);
         assert_ne!(at_foobar_index.generation, INVALID_GENERATION);
@@ -1180,13 +1385,22 @@ mod test {
     #[test]
     fn pool_iterator_mut_test() {
         let mut pool: Pool<String> = Pool::new();
-        let foobar = pool.spawn(format!("Foobar"));
-        let d = pool.spawn(format!("Foo"));
+        let foobar = pool.spawn("Foobar".to_string());
+        let d = pool.spawn("Foo".to_string());
         pool.free(d);
-        let baz = pool.spawn(format!("Baz"));
+        let baz = pool.spawn("Baz".to_string());
+        for s in pool.iter() {
+            println!("{}", s);
+        }
         for s in pool.iter_mut() {
             println!("{}", s);
         }
+        for s in &pool {
+            println!("{}", s);
+        }
+        for s in &mut pool {
+            println!("{}", s);
+        }
         pool.free(foobar);
         pool.free(baz);
     }
@@ -1200,16 +1414,39 @@ mod test {
 
         let mut pool = Pool::new();
         let foobar = pool.spawn(Value {
-            data: format!("Foobar"),
+            data: "Foobar".to_string(),
         });
         let bar = pool.spawn(Value {
-            data: format!("Bar"),
+            data: "Bar".to_string(),
         });
         let baz = pool.spawn(Value {
-            data: format!("Baz"),
+            data: "Baz".to_string(),
         });
         assert_eq!(pool.handle_of(pool.borrow(foobar)), foobar);
         assert_eq!(pool.handle_of(pool.borrow(bar)), bar);
         assert_eq!(pool.handle_of(pool.borrow(baz)), baz);
     }
+
+    #[test]
+    fn pool_test_spawn_at() {
+        let mut pool = Pool::new();
+
+        #[derive(Debug, Eq, PartialEq)]
+        struct Payload;
+
+        assert_eq!(pool.spawn_at(2, Payload), Ok(Handle::new(2, 1)));
+        assert_eq!(pool.spawn_at(2, Payload), Err(Payload));
+        assert_eq!(pool.records[0].payload, None);
+        assert_eq!(pool.records[1].payload, None);
+        assert_ne!(pool.records[2].payload, None);
+
+        assert_eq!(pool.spawn_at(2, Payload), Err(Payload));
+
+        pool.free(Handle::new(2, 1));
+
+        assert_eq!(pool.spawn_at(2, Payload), Ok(Handle::new(2, 2)));
+
+        assert_eq!(pool.spawn(Payload), Handle::new(1, 2));
+        assert_eq!(pool.spawn(Payload), Handle::new(0, 2));
+    }
 }
