@@ -12,7 +12,7 @@ pub use rg3d_core_derive::Visit;
 
 pub mod prelude {
     //! Types to use `#[derive(Visit)]`
-    pub use super::{Visit, VisitResult, Visitor};
+    pub use super::{Visit, VisitError, VisitResult, Visitor};
 }
 
 use crate::{
@@ -26,15 +26,17 @@ use crate::{
 };
 
 use byteorder::{LittleEndian, ReadBytesExt, WriteBytesExt};
+use fxhash::FxHashMap;
+use std::collections::HashMap;
 use std::{
     any::Any,
     cell::{Cell, RefCell},
-    collections::{hash_map::Entry, HashMap},
+    collections::hash_map::Entry,
     fmt::{Display, Formatter},
     fs::File,
     hash::Hash,
     io::{BufWriter, Cursor, Read, Write},
-    ops::DerefMut,
+    ops::{DerefMut, Range},
     path::{Path, PathBuf},
     rc::Rc,
     string::FromUtf8Error,
@@ -576,8 +578,7 @@ impl Field {
 
     fn load(file: &mut dyn Read) -> Result<Field, VisitError> {
         let name_len = file.read_u32::<LittleEndian>()? as usize;
-        let mut raw_name = Vec::with_capacity(name_len);
-        unsafe { raw_name.set_len(name_len) };
+        let mut raw_name = vec![Default::default(); name_len];
         file.read_exact(raw_name.as_mut_slice())?;
         let id = file.read_u8()?;
         Ok(Field::new(
@@ -615,8 +616,7 @@ impl Field {
                 }),
                 14 => FieldKind::Data({
                     let len = file.read_u32::<LittleEndian>()? as usize;
-                    let mut vec = Vec::with_capacity(len);
-                    unsafe { vec.set_len(len) };
+                    let mut vec = vec![Default::default(); len];
                     file.read_exact(vec.as_mut_slice())?;
                     vec
                 }),
@@ -710,8 +710,8 @@ impl Default for Node {
 
 pub struct Visitor {
     nodes: Pool<Node>,
-    rc_map: HashMap<u64, Rc<dyn Any>>,
-    arc_map: HashMap<u64, Arc<dyn Any + Send + Sync>>,
+    rc_map: FxHashMap<u64, Rc<dyn Any>>,
+    arc_map: FxHashMap<u64, Arc<dyn Any + Send + Sync>>,
     reading: bool,
     current_node: Handle<Node>,
     root: Handle<Node>,
@@ -735,8 +735,8 @@ impl Visitor {
         let root = nodes.spawn(Node::new("__ROOT__", Handle::NONE));
         Self {
             nodes,
-            rc_map: HashMap::new(),
-            arc_map: HashMap::new(),
+            rc_map: FxHashMap::default(),
+            arc_map: FxHashMap::default(),
             reading: false,
             current_node: root,
             root,
@@ -858,8 +858,7 @@ impl Visitor {
 
     fn load_node_binary(&mut self, file: &mut dyn Read) -> Result<Handle<Node>, VisitError> {
         let name_len = file.read_u32::<LittleEndian>()? as usize;
-        let mut raw_name = Vec::with_capacity(name_len);
-        unsafe { raw_name.set_len(name_len) };
+        let mut raw_name = vec![Default::default(); name_len];
         file.read_exact(raw_name.as_mut_slice())?;
 
         let mut node = Node {
@@ -1107,6 +1106,15 @@ where
     }
 }
 
+impl<T> Visit for parking_lot::Mutex<T>
+where
+    T: Visit + Send,
+{
+    fn visit(&mut self, name: &str, visitor: &mut Visitor) -> VisitResult {
+        self.get_mut().visit(name, visitor)
+    }
+}
+
 impl<T> Visit for Box<T>
 where
     T: Visit,
@@ -1283,7 +1291,53 @@ where
     }
 }
 
-impl<K, V, S: std::hash::BuildHasher> Visit for HashMap<K, V, S>
+impl<K, V> Visit for FxHashMap<K, V>
+where
+    K: Visit + Default + Clone + Hash + Eq,
+    V: Visit + Default,
+{
+    fn visit(&mut self, name: &str, visitor: &mut Visitor) -> VisitResult {
+        visitor.enter_region(name)?;
+
+        let mut count = self.len() as u32;
+        count.visit("Count", visitor)?;
+
+        if visitor.is_reading() {
+            for i in 0..(count as usize) {
+                let name = format!("Item{}", i);
+
+                visitor.enter_region(name.as_str())?;
+
+                let mut key = K::default();
+                key.visit("Key", visitor)?;
+
+                let mut value = V::default();
+                value.visit("Value", visitor)?;
+
+                self.insert(key, value);
+
+                visitor.leave_region()?;
+            }
+        } else {
+            for (i, (key, value)) in self.iter_mut().enumerate() {
+                let name = format!("Item{}", i);
+
+                visitor.enter_region(name.as_str())?;
+
+                let mut key = key.clone();
+                key.visit("Key", visitor)?;
+
+                value.visit("Value", visitor)?;
+
+                visitor.leave_region()?;
+            }
+        }
+
+        visitor.leave_region()
+    }
+}
+
+impl<K, V> Visit for HashMap<K, V>
 where
     K: Visit + Default + Clone + Hash + Eq,
     V: Visit + Default,
@@ -1365,6 +1419,17 @@ impl<T: Default + Visit, const SIZE: usize> Visit for [T; SIZE] {
     }
 }
 
+impl<T: Visit> Visit for Range<T> {
+    fn visit(&mut self, name: &str, visitor: &mut Visitor) -> VisitResult {
+        visitor.enter_region(name)?;
+
+        self.start.visit("Start", visitor)?;
+        self.end.visit("End", visitor)?;
+
+        visitor.leave_region()
+    }
+}
+
 #[cfg(test)]
 mod test {
     use crate::visitor::{Data, Visit, VisitError, VisitResult, Visitor};
@@ -1400,7 +1465,7 @@ mod test {
     impl Visit for ResourceKind {
         fn visit(&mut self, name: &str, visitor: &mut Visitor) -> VisitResult {
             match self {
-                ResourceKind::Unknown => Err(VisitError::User(format!("invalid resource type"))),
+                ResourceKind::Unknown => Err(VisitError::User("invalid resource type".to_string())),
                 ResourceKind::Texture(tex) => tex.visit(name, visitor),
                 ResourceKind::Model(model) => model.visit(name, visitor),
             }
@@ -1441,7 +1506,7 @@ mod test {
             } else {
                 let mut kind_id: u8 = match &self.kind {
                     ResourceKind::Unknown => {
-                        return Err(VisitError::User(format!("Invalid resource!")))
+                        return Err(VisitError::User("Invalid resource!".to_string()))
                     }
                     ResourceKind::Model(_) => 0,
                     ResourceKind::Texture(_) => 1,
@@ -1454,20 +1519,12 @@ mod test {
         }
     }
 
+    #[derive(Default)]
     struct Foo {
         bar: u64,
         shared_resource: Option<Rc<Resource>>,
     }
 
-    impl Default for Foo {
-        fn default() -> Self {
-            Self {
-                bar: 0,
-                shared_resource: None,
-            }
-        }
-    }
-
     impl Foo {
         fn new(resource: Rc<Resource>) -> Self {
             Self {
@@ -1502,7 +1559,7 @@ mod test {
 
             visitor.save_binary(path).unwrap();
             if let Ok(mut file) = File::create(Path::new("test.txt")) {
-                file.write(visitor.save_text().as_bytes()).unwrap();
+                file.write_all(visitor.save_text().as_bytes()).unwrap();
             }
         }
 
